@@ -1,4 +1,5 @@
 import pytest
+import django
 from django.db.models import IntegerField, TextField
 from django.db.models.aggregates import Count, Max, Min, Sum
 from django.db.models.expressions import (
@@ -668,3 +669,208 @@ class TestCTE(TestCase):
             {'amt': 32, 'region_name': 'earth', 'pnt_id': 'sun'},
             {'amt': 33, 'region_name': 'earth', 'pnt_id': 'sun'},
         ])
+
+    def test_django52_queryset_regression(self):
+        cte = CTE(Order.objects.values("id", "region_id"))
+        q = cte.queryset()
+        q.values("id", "region_id")  # Raises an exception before the fix
+
+    def test_django52_ambiguous_column_names(self):
+        cte = CTE(Order.objects.values("region", "amount", "user_id"))
+        cte2 = CTE(User.objects.annotate(user_id=F("id")), name="cte2")
+        qs = with_cte(
+            cte,
+            cte2,
+            select=cte2.join(cte.queryset(), user_id=cte2.col.user_id)
+            .annotate(user_name=cte2.col.name)
+            .order_by("region", "amount")
+            .values_list("region", "amount", "user_name"),
+        )
+        # Executing this query should not raise a
+        # django.db.utils.OperationalError: ambiguous column name: user_id
+        self.assertEqual(list(qs), [
+            ('earth', 30, "admin"),
+            ('earth', 31, "admin"),
+            ('earth', 32, "admin"),
+            ('earth', 33, "admin"),
+            ('mars', 40, "admin"),
+            ('mars', 41, "admin"),
+            ('mars', 42, "admin"),
+            ('mercury', 10, "admin"),
+            ('mercury', 11, "admin"),
+            ('mercury', 12, "admin"),
+            ('moon', 1, "admin"),
+            ('moon', 2, "admin"),
+            ('moon', 3, "admin"),
+            ('proxima centauri', 2000, "admin"),
+            ('proxima centauri b', 10, "admin"),
+            ('proxima centauri b', 11, "admin"),
+            ('proxima centauri b', 12, "admin"),
+            ('sun', 1000, "admin"),
+            ('venus', 20, "admin"),
+            ('venus', 21, "admin"),
+            ('venus', 22, "admin"),
+            ('venus', 23, "admin"),
+        ])
+
+    def test_django52_queryset_aggregates_klass_error(self):
+        cte = CTE(
+            Order.objects.annotate(user_name=F("user__name"))
+            .values("user_name")
+            .annotate(c=Count("user_name"))
+            .values("user_name", "c")
+        )
+        qs = with_cte(cte, select=cte)
+        # Executing the query should not raise TypeError: 'NoneType' object is not subscriptable
+        self.assertEqual(list(qs), [{"user_name": "admin", "c": 22}])
+
+    def test_django52_annotate_model_field_name_after_queryset(self):
+        # Select the `id` field in one CTE
+        cte = CTE(Order.objects.values("id", "region", "user_id"))
+        # In the next query, when querying from the CTE we reassign the `id` field
+        # Previously, this would have thrown an exception
+        qs = (
+            with_cte(cte, select=cte)
+            .annotate(id=F('user_id'))
+            .values_list('id', 'region')
+            .order_by('id', 'region')
+            .distinct()
+        )
+        self.assertEqual(list(qs), [
+            (1, 'earth'),
+            (1, 'mars'),
+            (1, 'mercury'),
+            (1, 'moon'),
+            (1, 'proxima centauri'),
+            (1, 'proxima centauri b'),
+            (1, 'sun'),
+            (1, 'venus'),
+        ])
+
+    @pytest.mark.skipif(django.VERSION < (5, 2), reason="Requires Django 5.2+")
+    def test_queryset_after_values_list(self):
+        cte = CTE(Order.objects.values_list("region", "amount").order_by("region", "amount"))
+        qs = with_cte(cte, select=cte)
+        self.assertEqual(list(qs), [
+            ('earth', 30),
+            ('earth', 31),
+            ('earth', 32),
+            ('earth', 33),
+            ('mars', 40),
+            ('mars', 41),
+            ('mars', 42),
+            ('mercury', 10),
+            ('mercury', 11),
+            ('mercury', 12),
+            ('moon', 1),
+            ('moon', 2),
+            ('moon', 3),
+            ('proxima centauri', 2000),
+            ('proxima centauri b', 10),
+            ('proxima centauri b', 11),
+            ('proxima centauri b', 12),
+            ('sun', 1000),
+            ('venus', 20),
+            ('venus', 21),
+            ('venus', 22),
+            ('venus', 23),
+        ])
+
+    @pytest.mark.skipif(django.VERSION < (5, 2), reason="Requires Django 5.2+")
+    def test_queryset_after_values_list_flat(self):
+        cte = CTE(
+            Order.objects.values_list("region", flat=True)
+            .order_by("region")
+            .distinct()
+        )
+        qs = with_cte(cte, select=cte)
+        self.assertEqual(list(qs), [
+            'earth',
+            'mars',
+            'mercury',
+            'moon',
+            'proxima centauri',
+            'proxima centauri b',
+            'sun',
+            'venus'
+        ])
+
+    @pytest.mark.skipif(django.VERSION < (5, 2), reason="Requires Django 5.2+")
+    def test_queryset_values_list_order1(self):
+        cte = CTE(
+            Order.objects.values("region")
+            .annotate(c=Count("region"))
+            .values_list("c", "region")
+            .order_by("region")
+        )
+        qs = with_cte(cte, select=cte)
+        # Ensure the column order of queried fields is the specified one: c, region
+        # Before the fix, the order would have been this one: region, c
+        self.assertEqual(list(qs), [
+            (4, 'earth'),
+            (3, 'mars'),
+            (3, 'mercury'),
+            (3, 'moon'),
+            (1, 'proxima centauri'),
+            (3, 'proxima centauri b'),
+            (1, 'sun'),
+            (4, 'venus'),
+        ])
+
+    @pytest.mark.skipif(django.VERSION < (5, 2), reason="Requires Django 5.2+")
+    def test_queryset_values_list_order2(self):
+        cte = CTE(
+            Order.objects.values("region")
+            .annotate(r=F("region"), c=Count("region"))
+            .values_list("c", "r")
+            .order_by("r")
+        )
+        qs = with_cte(cte, select=cte)
+        # Ensure the column order of queried fields is the specified one: c, r
+        # Before the fix, the order would have been this one: r, c
+        self.assertEqual(list(qs), [
+            (4, 'earth'),
+            (3, 'mars'),
+            (3, 'mercury'),
+            (3, 'moon'),
+            (1, 'proxima centauri'),
+            (3, 'proxima centauri b'),
+            (1, 'sun'),
+            (4, 'venus'),
+        ])
+
+    @pytest.mark.skipif(django.VERSION < (5, 2), reason="Requires Django 5.2+")
+    def test_left_outer_join_invalid_innerjoin(self):
+        totals = CTE(
+            Order.objects
+            .values("region_id")
+            .annotate(total=Sum("amount"))
+        )
+        # Query all regions but only show the total order amount for regions that are
+        # grandchildren of "sun".
+        # This requires the __parent to be joined using LEFT OUTER JOIN too.
+        # Otherwise, if django did an INNER JOIN for __parent then only regions that
+        # have a parent would be included in the result set.
+        qs = with_cte(
+            totals,
+            select=totals.join(
+                Region,
+                parent__parent="sun",
+                name=totals.col.region_id,
+                _join_type=LOUTER
+            ).values("name", total=totals.col.total)
+            .order_by("name")
+        )
+        self.assertEqual(list(qs), [
+            {'name': "bernard's star", 'total': None},
+            {'name': 'deimos', 'total': None},
+            {'name': 'earth', 'total': None},
+            {'name': 'mars', 'total': None},
+            {'name': 'mercury', 'total': None},
+            {'name': 'moon', 'total': 6},
+            {'name': 'phobos', 'total': None},
+            {'name': 'proxima centauri', 'total': None},
+            {'name': 'proxima centauri b', 'total': None},
+            {'name': 'sun', 'total': None},
+            {'name': 'venus', 'total': None}
+        ])
